-- To do: Add complete parsing.

with Ada.Characters.Handling;

package body CGI.Characters is

      function To_Decimal(C : in Character) return Natural is
      begin
        case C is
          when '0' ..'9' => return Character'Pos(C) - Character'Pos('0');
          when 'A'..'F'  => return Character'Pos(C) - Character'Pos('A') + 10;
          when 'a'..'f'  => return Character'Pos(C) - Character'Pos('a') + 10;
          when others => raise Program_Error;
        end case;
      end;

    function Hex_Value(H : in String) return Character is
      -- Given hex string, return its Value as a Natural.
      Value : Natural := 0;
    begin
      for P in H'Range loop
        Value := Value * 16 + To_Decimal(H(P));
      end loop;
     
      return Character'Val(Value);
    end Hex_Value;
    
    Hex_Digit : constant array (0..15) of Character
        := ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
    function Hex_Image (C : in Character) return String is
      Value : constant Natural := Character'Pos(C);
    begin
      return String'(1 => Hex_Digit(Value/16),
                     2 => Hex_Digit(Value rem 16));
    end Hex_Image;

  function Decode (Encoded : in String) return String is
    -- Decoding will always shrink the string, so allocate
    -- the result as large as the Encoded string and start
    -- filling it up. Then return the slice of what is used.
    
    -- To handle the incompatibility between Mosaic and Netscape,
    -- a second pass decodes what Mosaic left.

    -- In the given string, convert pattern %HH into alphanumeric characters,
    -- where HH is a hex number. Since this encoding only permits values
    -- from %00 to %FF, there's no need to handle 16-bit characters.
    Result : String  := Encoded;
    Last   : Natural := Result'Last;
    Pos    : Natural;
    procedure Check_and_Replace (Pattern : in String; -- not including '&' at Pos
                                 Char    : in Character) is
      Length : constant Natural := Pattern'Length;
    begin
      if (Last - Pos) >= Length and then
         Result(Pos+1..Pos+Length) = Pattern then -- Found!
        Result(Pos) := Char;
        Result(Pos+1.. Last-Length) := Result(Pos+Length+1 ..Last);
        Last := Last -Length;
      end if;
    end Check_and_Replace;
  begin
    First_Pass: -- Convert + to ' '
    for I in Result'Range loop
      if Result(I) = '+' then Result(I) := ' ';
      end if;
    end loop First_Pass;
    
    Pos := Result'First;
    Second_Pass: -- Convert Hexadecimal Patterns
    while Pos < Last-1 loop
      if Result(Pos) = '%'                  and then
        Ada.Characters.Handling.Is_Hexadecimal_Digit(Result(Pos+1)) and then
        Ada.Characters.Handling.Is_Hexadecimal_Digit(Result(Pos+2))
      then
        Result(Pos) := Hex_Value(Result(Pos+1..Pos+2));
        Result(Pos+1..Last-2) := Result(Pos+3..Last);
        Last := Last -2;
      end if;
      Pos := Pos +1;
    end loop Second_Pass;
      
    Pos := Result'First;
    Third_Pass: -- Convert Remaining Patterns
    while Pos < Last-2 loop
      if Result(Pos) = '&' then
        Check_and_Replace("#32;",' ');
        Check_and_Replace("gt;",'>');
        Check_and_Replace("quot;",'"');
        Check_and_Replace("amp;",'&');
        Check_and_Replace("lt;",'<');
      end if;
      Pos := Pos +1;
    end loop Third_Pass;
      
    return Result(Result'First..Last);

  end Decode;
  
  function Encode (HTML    : in String) return String is
    -- For Mosaic and Netscape, it is sufficient to encode the following:
--    Space : constant String := "&#32;";
    LT    : constant String := "&lt;";
    GT    : constant String := "&gt;";
    Quote : constant String := "&quot;";

    -- avoid recursion and lots of special string handling by
    -- going through and counting special characters first to
    -- determine the length of the result. Then allocate the
    -- result string and fill it in.
    Length : Natural := 0;
  begin
    Count_Special_Characters:
    for C in HTML'Range loop
      case HTML(C) is
--        when ' '    => Length := Length + Space'Length;
        when '<'    => Length := Length + LT'Length;
        when '>'    => Length := Length + GT'Length;
        when '"'    => Length := Length + Quote'Length;
        when others => Length := Length + 1;
      end case;
    end loop Count_Special_Characters;
    
    Build_Result:
    declare
      Result : String (1..Length);
      Cursor : Natural := Result'First;
      procedure Insert (S : in String) is
      begin
        Result(Cursor..Cursor+S'Length-1) := S;
        Cursor := Cursor + S'Length;
      end;
    begin
      for C in HTML'Range loop
        case HTML(C) is
--          when ' '    => Insert (Space);
          when '<'    => Insert (LT);
          when '>'    => Insert (GT);
          when '"'    => Insert (Quote);
          when others => Result(Cursor) := HTML(C);
                         Cursor := Cursor +1;
        end case;
      end loop;
      
      return Result;
    end Build_Result;
  end Encode;

end CGI.Characters;
