package CGI.Parameters is  -- "http://hoohoo.ncsa.uiuc.edu/cgi/".
  -- This package splits the input into a set of variables that can be accessed
  -- by position or by name.
  -- An "Isindex" request is translated into a request with a single
  -- key named "isindex" with its Value as the query value.
  
  -- This package provides CGI Parameters two ways:
  -- 1) Associative access; provide the key name and the
  --    value associated with that key will be returned.
  -- 2) As a sequence of key-value pairs, indexed from 1 to Argument_Count.
  --    This is similar to the Ada pre-defined package Ada.Command_Line.
  -------------------------------------------------------------------
  function Parsing_Errors return Boolean;  -- True if Error on Parse.
  function Input_Received return Boolean;  -- True if Input Received.
  function Is_Index       return Boolean;  -- True if an Isindex request made.

  -------------------------------------------------------------------
  -- Access data as an associative array - given a key, return its value.
  -- The Key value is case-sensitive.
  -- If a key is not present, Value raises Constraint_Error;
  function Value       (Key      : in String;
                        Index    : in Positive := 1) return String;
  function Key_Exists  (Key      : in String;
                        Index    : in Positive := 1) return Boolean;
  function Value_Count (Key      : in String)        return Natural;
  
  generic
     with procedure Evaluate (Value : in String);
  procedure Iterate_Key (Key : in String);
  
  
  -------------------------------------------------------------------
  -- Access data as an ordered list (it was sent as Key=Value);
  -- Keys and Values may be retrieved from Position (1 .. Argument_Count).
  -- Constraint_Error will be raised if Position > Argument_Count
  function Argument_Count                return Natural; -- 0 means no data sent
  function Key  (Position : in Positive) return String;
  function Value(Position : in Positive) return String;
  
  generic
     with procedure Evaluate (Key   : in String;
                              Value : in String);
  procedure Iterate;

end CGI.Parameters;

---------------------
-- Derived from initial work by David A. Wheeler (wheeler@ida.org) June 1995.
-- Written by D. Douglas Smith (AdaSmith@ACM.org) February 1996.

-- This was inspired by <a href="http://www.bio.cam.ac.uk/web/form.html">a perl binding</a> by Steven E. Brenner.
-- and <a href="http://www-genome.wi.mit.edu/ftp/pub/software/WWW/cgi_docs.html">another perl binding</a> by L. Stein.

-- There is a different method for interfacing binding Ada with CGI, <a href="http://www.hyperion.com/~koreth/uncgi.html">Un-CGI</a>.
  
