package body HTML is
  generic
    Prefix : in String;
    Suffix : in String;
  function Encase(Text : in String) return String;
  function Encase(Text : in String) return String is
  begin
    return Prefix & Text & Suffix;
  end;

  function Start_HTML               return String is
  begin
    return "<html>";
  end;
  
  function Head_Imp  is new Encase ("<head>","</head>");
  function Head (Text : in String) return String renames Head_Imp;

  function Title_Imp is new Encase ("<title>","</title>");
  function Title (Text : in String) return String renames Title_Imp;

  function Start_Body               return String is
  begin
    return "<body>";
  end;

  generic
    type T is (<>);
    Name : in String;
    Default : in T := T'First;
  function If_Not_Default_Attribute (Value : in T) return String;
  function If_Not_Default_Attribute (Value : in T) return String is
  begin
    if Value /= Default then
      return Name & T'Image(Value);
    else
      return "";
    end if;
  end;
  
  -- Header --------
  function If_Not_Left is
    new If_Not_Default_Attribute (H_Align, " align=", Left);
    
  function Header (Text  : in String;
                   Level : in Natural := 1;
                   Align : in H_Align := Left) return String is
    HL : String := Natural'Image(Level);
  begin
    HL(HL'First) := 'h';
    return '<' & HL & If_Not_Left(Align) & '>' & Text & "</" & HL & '>';
  end;

  -- Style ---------
  function Bold_Imp      is new Encase ("<b>","</b>");
  function Bold (Text : in String) return String renames Bold_Imp;

  function Italics_Imp   is new Encase ("<i>","</i>");
  function Italics (Text : in String) return String renames Italics_Imp;

  function Paragraph                    return String is
  begin
    return "<p>";
  end;

  -- Definition ----
  function Start_Def_List return String is
  begin
    return "<dl>";
  end;
  
  function Define (Term       : in String;
                   Definition : in String) return String is
  begin
    return "<dt>" & Term & "<dd>" & Definition;
  end;
  
  function End_Def_List return String is
  begin
    return "</dl>";
  end;


  -- Table ---------
  function Table (Border : in Boolean := False) return String is
  begin
    return "<table>";
  end;

  function Row return String is
  begin
    return "<tr>";
  end;

  function If_Not_One (Name  : in String;
                       Value : in Natural) return String is
  begin
    if Value /= 1 then
      return ' ' & Name & "=""" & Natural'Image(Value) & '"';
    else
      return "";
    end if;
  end;
  function Table_Header (Text     : in String;
                         Span_Row : in Natural := 1;
                         Span_Col : in Natural := 1) return String is
  begin
    return "<th" & If_Not_One("span_row", Span_Row) &
                   If_Not_One("span_col", Span_Col) &
                   '>' & Text & "</th>";
  end;
  
  function Cell (Text     : in String;
                 Span_Row : in Natural := 1;
                 Span_Col : in Natural := 1) return String is
  begin
    return "<td" & If_Not_One("span_row",Span_Row) &
                   If_Not_One("span_col",Span_Col) &
                   '>' & Text & "</td>";
  end;
  function End_Table return String is
  begin
    return "</table>";
  end;

  function End_Body_HTML return String is
  begin
    return "</body></html>";
  end;

end HTML;
